#include "circuits.h"

void sorter(Circuit * circ, lit_t a, lit_t b, lit_t * low, lit_t * high)
{
  assert(a != b);
  assert(lit_to_var(a) <= circ->max_num);
  assert(lit_to_var(b) <= circ->max_num);
  *low = circ->max_num + 1;
  *high = circ->max_num + 2;
  Gate * low_gate = circuit_add_gate(circ, *low, GATE_AND);
  Gate * high_gate = circuit_add_gate(circ, *high, GATE_OR);

  circuit_add_to_gate(circ, low_gate, a);
  circuit_add_to_gate(circ, low_gate, b);
  circuit_add_to_gate(circ, high_gate, a);
  circuit_add_to_gate(circ, high_gate, b);
}

GArray * sort_network(Circuit * circ, GArray * lits, size_t first, size_t last)
{
  size_t sortees = last - first;

  assert(sortees > 0);

  if (sortees == 1) {
    GArray * result = g_array_sized_new(FALSE, FALSE, sizeof(lit_t), 1);
    lit_t l = g_array_index(lits, lit_t, first);

    g_array_append_val(result, l);

    return result;
  }
  else if (sortees == 2) {
    GArray * result = g_array_sized_new(FALSE, FALSE, sizeof(lit_t), 2);
    lit_t a = g_array_index(lits, lit_t, first);
    lit_t b = g_array_index(lits, lit_t, first + 1);
    lit_t low, high;

    sorter(circ, a, b, &low, &high);
    g_array_append_val(result, low);
    g_array_append_val(result, high);

    return result;
  }
  else {
    size_t middle = (last - first + 1)  / 2;
    GArray * top = sort_network(circ, lits, first, first + middle);
    GArray * bottom = sort_network(circ, lits, first + middle, last);
    GArray * sorted = merge_network(circ, top, bottom);

    g_array_unref(top);
    g_array_unref(bottom);

    return sorted;
  }
}

GArray * merge_network(Circuit * circ, GArray * a, GArray * b)
{
  assert(a->len >= b->len);
  assert((a->len - b->len) < 2);
  size_t result_length = a->len + b->len;
  GArray * even = g_array_sized_new(FALSE, FALSE, sizeof(lit_t), (result_length + 1) / 2);
  GArray * odd = g_array_sized_new(FALSE, FALSE, sizeof(lit_t), result_length / 2);
  GArray * result = g_array_sized_new(FALSE, FALSE, sizeof(lit_t), result_length);

  for (int i = 0; i < result_length; i++) {
    lit_t alit, blit;

    switch(i % 4) {
    case 0:
      alit = g_array_index(a, lit_t, i / 2);
      g_array_append_val(even, alit);
      break;
    case 1:
      blit = g_array_index(b, lit_t, i / 2);
      g_array_append_val(even, blit);
      break;
    case 2:
      alit = g_array_index(a, lit_t, i / 2);
      g_array_append_val(odd, alit);
      break;
    case 3:
      blit = g_array_index(b, lit_t, i / 2);
      g_array_append_val(odd, blit);
      break;
    }
  }

  GArray * sorted_even = sort_network(circ, even, 0, even->len);
  GArray * sorted_odd = sort_network(circ, odd, 0, odd->len);
  lit_t first = g_array_index(sorted_even, lit_t, 0);
  int i;

  g_array_unref(even);
  g_array_unref(odd);
  g_array_append_val(result, first);

  for (i = 0; i < sorted_even->len - 1; i++) {
    lit_t low, high;
    lit_t elit = g_array_index(sorted_even, lit_t, i + 1);

    if (i < sorted_odd->len) {
      lit_t olit = g_array_index(sorted_odd, lit_t, i);

      sorter(circ, elit, olit, &low, &high);
      g_array_append_val(result, low);
      g_array_append_val(result, high);
    }
    else {
      g_array_append_val(result, elit);
    }
  }

  if (i < sorted_odd->len) {
    lit_t lastodd = g_array_index(sorted_odd, lit_t, i);

    g_array_append_val(result, lastodd);
  }

  g_array_unref(sorted_even);
  g_array_unref(sorted_odd);

  return result;
}

var_t card_network(Circuit * circ, GArray * lits, size_t k)
{
  GArray * sorted = sort_network(circ, lits, 0, lits->len);
  var_t indicator = g_array_index(sorted, var_t, lits->len - 1 - k);

  g_array_unref(sorted);

  return create_lit(indicator, false);
}
