import sys
import re
import tempfile
import argparse
import subprocess
import os

import qbfolkman
import qdimacs

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Turn Codish's canon permutations into CNF clauses")

    parser.add_argument("bee", help="Location of the BEE directory")
    parser.add_argument("canonfile", help="Canonizing permutaitons", type=argparse.FileType('r'))

    opt = parser.parse_args()
    isotext = opt.canonfile.read()
    infomatch = re.search("canonizing_set\((\d+),", isotext)

    opt.canonfile.close()
    assert(infomatch is not None)

    order = int(infomatch.group(1))
    uppertriag = []

    with tempfile.NamedTemporaryFile(mode="w") as beefile:
        for i in range(0, order):
            for j in range(i + 1, order):
                # uppertriag.append(chr(96 + qbfolkman.edge(i, j, order)))
                uppertriag.append("e%d" % qbfolkman.edge(i, j, order))
                print("new_bool(e%d)" % qbfolkman.edge(i, j, order), file=beefile)

        for i in re.finditer("\[[\d\,]+]", isotext):
            perm = [int(j) - 1 for j in i.group(0).strip("[]").split(",")]
            permuppertriag = []

            for i in range(0, order):
                for j in range(i + 1, order):
                    # permuppertriag.append(chr(96 + qbfolkman.edge(perm[i], perm[j], order)))
                    permuppertriag.append("e%d" % qbfolkman.edge(perm[i], perm[j], order))

            print("bool_arrays_lex([%s], [%s])" % (",".join(uppertriag), ",".join(permuppertriag)), file=beefile)

        print("solve satisfy()", file=beefile)

        beefile.flush()
        subprocess.check_call(["./BumbleBEE", beefile.name, "-dimacs", "tmp.cnf", "tmp.map"], cwd=opt.bee, stdout=subprocess.DEVNULL, universal_newlines=True)

        with open(os.path.join(opt.bee, "tmp.cnf"), 'r') as tmpcnf:
            tmpformula, nvars = qdimacs.read_dimacs(tmpcnf)

            with open(os.path.join(opt.bee, "tmp.map"), 'r') as tmpmap:
                badmap = {}
                mapre = re.compile("\(e(\d+),bool,(\d+)\)\.")
                maxvar = order * (order - 1) // 2

                for l in tmpmap:
                    mapmatch = mapre.match(l.strip())

                    if mapmatch is not None:
                        badmap[int(mapmatch.group(2))] = int(mapmatch.group(1))
                        badmap[-int(mapmatch.group(2))] = -int(mapmatch.group(1))

                assert(len(badmap) == 2 * maxvar)
                assert(max(badmap.values()) == maxvar)

                for i in range(0, nvars):
                    if (i + 1) not in badmap:
                        maxvar += 1
                        badmap[i + 1] = maxvar
                        badmap[-(i + 1)] = -maxvar

                print("p cnf %d %d" % (nvars, len(tmpformula)))

                for c in tmpformula:
                    print("%s 0" % " ".join([str(badmap[l]) for l in c]))

        os.unlink(os.path.join(opt.bee, "tmp.cnf"))
        os.unlink(os.path.join(opt.bee, "tmp.map"))
