import sys
import re
import subprocess
import tempfile

import generate_experiments
import qcir
import s2s

def parse_perf(perfrun):
    res = perfrun.wait()

    if res == 124:
        return None
    else:
        cpuclockline = ""

        while "cpu-clock" not in cpuclockline:
            cpuclockline = perfrun.stderr.readline().strip()

        runinfo = cpuclockline.split(",")

        return float(runinfo[0])

if __name__ == "__main__":
    maxorder = int(sys.argv[1])
    bee = sys.argv[2]
    are = re.compile("a[^\s+] = (\d+)")
    a = []
    preva = set()
    timeoutargs = ["timeout", "600"]
    perfargs = ["perf", "stat", "-C", "0", "-x,", "-r5"]
    cgexecargs = ["cgexec", "-g", "cpuset,memory:qbfolkman"]
    testargs = timeoutargs + perfargs + cgexecargs

    with tempfile.NamedTemporaryFile(mode="w") as encoding:
        generate_experiments.test_generation_encoding(encoding, maxorder)
        encoding.flush()

        with subprocess.Popen(["./BumbleBEE", encoding.name], cwd=bee, stdout=subprocess.PIPE, universal_newlines=True) as BumbleBEE:
            for l in BumbleBEE.stdout:
                l = l.strip()

                if l == "----------":
                    if tuple(sorted(a)) not in preva:
                        preva.add(tuple(sorted(a)))

                    a = []
                else:
                    m = are.match(l)

                    if m is not None:
                        aval = int(m.group(1))

                        if aval > 0:
                            a.append(aval)

    print("About to test %d cases" % len(preva), file=sys.stderr)

    for a in sorted(preva, key=lambda x: (sum([i - 1 for i in x]) + max(x), len(x))):
        runline = ["\\texttt{%s}" % ",".join([str(i) for i in a])]

        # First run s2s.py
        with tempfile.NamedTemporaryFile(mode="w") as outfile:
            s2s.s2s(a, outfile)
            outfile.flush()
            
            with subprocess.Popen(testargs + ["./sat-to-sat", "-n=0", outfile.name], stderr=subprocess.PIPE, stdout=subprocess.DEVNULL, universal_newlines=True) as s2srun:
                t = parse_perf(s2srun)
                
                runline.append("%.2f" % t if t is not None else "-")

        # Then run qdimacs
        with subprocess.Popen(testargs + ["../build/qdimacs", "--PG"] + [str(i) for i in a], stderr=subprocess.PIPE, stdout=subprocess.DEVNULL, universal_newlines=True) as qdimacsrun:
            t = parse_perf(qdimacsrun)

            runline.append("%.2f" % t if t is not None else "-")

        # Finally generate a qcir encoding and run qfun
        with tempfile.NamedTemporaryFile(mode="w") as outfile:
            qcir.qcir(a, outfile)
            outfile.flush()

            with subprocess.Popen(testargs + ["../qfun/qfun", "-e", outfile.name], stderr=subprocess.PIPE, stdout=subprocess.DEVNULL, universal_newlines=True) as qfunrun:
                t = parse_perf(qfunrun)

                runline.append("%.2f" % t if t is not None else "-")

        print("&".join(runline))
