import re

import igraph

def nchoosek(n, k):
    res = 1

    for i in range(n, n - k, -1):
        res *= i

    for i in range(1, k + 1):
        res //= i

    return res

def edge(i, j, n):
    ii = max(i, j)
    jj = min(i, j)

    return jj * (n - 1) - (jj * (jj - 1) // 2) + (ii - jj)

def vcolor(v, c, n, lastid):
    return lastid + c * n + v + 1

def parameters(a):
    m = sum([(i - 1) for i in a]) + 1
    ar = sorted(a)[-1]

    return m, ar

def correct_answer(a):
    m, ar = parameters(a)
    g = igraph.Graph.Full(m + ar)

    g.delete_edges([(i, i + 1) for i in range(0, 2 * ar)])
    g.delete_edges([(0, 2 * ar)])

    return g

def read_dimacs(cnffile, ignore_header=False):
    header = cnffile.readline()
    nvars = 0

    while header.startswith("c"):
        header = cnffile.readline()

    headerinfo = re.match("p cnf (\d+) (\d+)\n", header)
    formula = []

    assert(ignore_header or (headerinfo is not None))

    if (headerinfo is not None) and not ignore_header:
        nvars = int(headerinfo.group(1))

    for l in cnffile:
        l = l.strip()

        if not l.startswith("c"):
            assert(l.endswith(" 0"))
            clause = [int(v) for v in l[:-len(" 0")].split(" ")]

            formula.append(clause)

            if ignore_header:
                nvars = max(nvars, max(clause))

    return (formula, nvars)
