#include "../circuits.h"

#include <gsl/gsl_combination.h>

int main(int argc, char ** argv)
{
  if (argc != 3) {
    return -1;
  }

  size_t ninputs = atol(argv[1]);
  size_t nones = atol(argv[2]);
  Circuit * circ = circuit_init();
  Scope * exists = circuit_init_scope(circ, QUANT_EXISTS);
  GArray * inputs = g_array_sized_new(FALSE, FALSE, sizeof(lit_t), ninputs);

  for (int i = 1; i <= ninputs; i++) {
    circuit_new_var(circ, exists, i);
    g_array_append_val(inputs, i);
  }

  for (int i = 0; i < ninputs; i++) {
    lit_t indicator = card_network(circ, inputs, i);
    gsl_combination * ones = gsl_combination_calloc(ninputs, nones);

    do {
      for (int i = 0; i < ninputs; i++) {
        circuit_set_value(circ, i + 1, 0);
      }

      for (size_t i = 0; i < nones; i++) {
        circuit_set_value(circ, gsl_combination_get(ones, i) + 1, 1);
      }

      circuit_evaluate(circ);

      assert(circuit_get_value(circ, indicator) == ((nones <= i) ? 0 : 1));
    } while (gsl_combination_next(ones) == GSL_SUCCESS);

    gsl_combination_free(ones);
  }

  g_array_unref(inputs);
}
